/****************************************************************************\
|* nettool.h -- Network Toolkit Library V1.0a by Steve Conley               *|
|* (c)1997 Project INSANITY Software, All Rights Reserved.                  *|
\****************************************************************************/

#ifndef __NETTOOL__H__
#define __NETTOOL__H__

#include <netinet/in.h>

/* If bzero doesn't exist, define it */
#ifndef bzero
#define bzero(s,n)	memset(s,0,n);
#endif

/* Show output from library?  Define QUIET_LIBRARY if you want it to keep
 * quiet.
 */
#undef QUIET_LIBRARY

/* How many connections should `listen' queue?
 */
#define QUEUE_LEN 10

/* Maximum hostname length allowed */
#define MAXHOSTNAME  50

/* This shouldn't be accessed by the user.  It's the actual connections
 * list, where all the user's connections are.
 */
typedef struct incomming_con_info{
	int    descr;					/* Descriptor num */
	char   host[MAXHOSTNAME+1];		/* Hostname       */
	struct incomming_con_info* next;	/* Next structure */
} CON;

/* Type INPCON -- INPut CONnection.  Accepts connection to a master port
 */
typedef struct {
	int  port;			/* Master port number       */
	int  master_socket;		/* Master socket/descriptor */
	int  numconnects;		/* Number connected         */
	CON* head;			/* Head of connections list */
} INPCON;

/* Type OUTCON -- OUTput CONnection.  Connects to a remote server's port.
 */
typedef struct {
	int descr;			/* Descriptor of connection    */
} OUTCON;

/* Input connection initialization function.  Takes a port and returns the
 * structure refering to it.  Handles all book-keeping and such required.
 */
INPCON* inpcon_init(int port);

/* This accepts a single connection -- run it multiple times if you want to
 * accept more people.  This should be checked rather regularly; it only
 * accepts one connection to make new connections easier to handle, though
 * you will need to check this pretty frequently to make sure you don't 
 * loose connections.  Returns 1 if someone's connected, 0 otherwise.
 */
int     inpcon_accept(INPCON* connections);

/* This takes a connection and sends a message over it.  Returns 1 on success,
 * 0 otherwise.  This function attempts to recover/garbage collect if the
 * write fails.  For example, if someone's connection drops and you write to
 * that connection, if will try to kill the descriptor and prevent the program
 * from crashing due to a broken pipe error. >8P
 */
int     consend(CON* connection,char* message);

/* This takes a connection and returns what's on its input queue, if anything.
 * It's NULL if nothing was on the queue.  The size of the buffer is between
 * 0 and 8192 bytes long, depending on how long the message itself was.  It
 * allocates an 8k buffer once, but tries to make the output as small as
 * possible.
 */
char*   conrecv(CON* connection);

/* This will disconnect a connection.  It returns 1 on success, 0 otherwise.
 * It cleans up all the memory used by the structure.
 */
int     disconnect(INPCON* list,CON* connection);

/* This disconnects and deallocates all memory for an input connection
 */
void    INPdestroy(INPCON* table);

/***************************************************************************/
/* Output connection functions below, input connection functions above     */
/***************************************************************************/

/* This opens an output connection.  Provide a hostname and a port, and this
 * will connect -- or try to connect -- to that location.
 */
OUTCON* outcon_init(char* host,char* port);

/* This attempts to write something to that previously connected socket.
 * It returns 1 on success, 0 otherwise.  It attempts to snag any signals
 * that might make the program crash (ie broken pipes usually)
 */
int     outcon_write(OUTCON* connection,char *msg);

/* This attempts to read something from a connection.  It returns either the
 * string recieved or NULL, depending on if there's something there or not.
 * It can read up to 8k at a time from a connection.
 */
char*   outcon_read(OUTCON* connection);

/* Destroy an output connection, thus disconnecting it and deallocating the
 * memory used.
 */
void    OUTdestroy(OUTCON* connection);

#endif /* End of library */
