/************************************************************************\
|* outcon.c -- NetTool output connection source code.                   *|
|* (c)1997 Project INSANITY Software.  By: Steve Conley.                *|
\************************************************************************/

/* System includes */
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <unistd.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <signal.h>

/* Project includes */
#include "nettool.h"

/* Project functions */
int nonblock(int s);
int descrwrite(int desc,char* text);
int open_connection(char *host, int port);

/* Write to the connection */
int outcon_write(OUTCON* connection,char* msg){
	return descrwrite(connection->descr,msg);
}

/* Read from the connection */
char* outcon_read(OUTCON* connection){
	char  buffer[8192];
	char* buf;
	int   length;
	bzero(buffer,8192);
	length=read(connection->descr,buffer,8192);
	if(length==-1){
		return NULL;
	}
	buffer[8191]='\0';
	buf=(char*)malloc(strlen(buffer)+1);
	strcpy(buf,buffer);
	return(buf);
}

/* Kill off the connection and free it up */
void OUTdestroy(OUTCON* connection){
	close(connection->descr);
	free(connection);
}


/* Initialize a output connection.  Some of this code was contributed by
 * Lion Templin -- Thanks for his help!
 */
OUTCON* outcon_init(char* host,char* port){
/* Set up our variables */
	OUTCON* newcon;
	int     portnum;
	struct  servent* serv;

/* Allocate memory and start setting up the address structure. */
	newcon=(OUTCON*)malloc(sizeof(OUTCON));

/* Is the provided port a number or a service name? */
	if(isdigit(*port)){
		portnum=atoi(port); /* If it's a number, grab it */
	}else{ /* Otherwise, get the service from /etc/services */
		serv=getservbyname(port,"tcp");
		if(serv==NULL){ /* Was it there? */
#ifndef QUIET_LIBRARY
			printf("Nettool: outcon_init: Invalid service name.\n");
#endif
			return (OUTCON*) NULL; /* Nope! >:P */
		}
		portnum=serv->s_port; /* Otherwise, we have it */
	}
/* This uses Lion Templin's function to connect to the server and return
 * a socket.
 */
	newcon->descr=open_connection(host,portnum);
	nonblock(newcon->descr);
	if(!newcon->descr){
		return (OUTCON*) NULL;
	}
	return(newcon);
}

int open_connection(char *host, int port){
	int ip1, ip2, ip3, ip4, len, opt;
	int sock;
	long ipaddr;
	struct sockaddr_in server;
	opt = 1;
	server.sin_addr.s_addr = inet_addr(host);
	if (server.sin_addr.s_addr == -1){
		struct hostent *hp;
		if ((hp = gethostbyname(host)) != NULL){
			memset((char *)&server, 0, sizeof(server));
			memcpy((char *)&server.sin_addr, hp->h_addr, hp->h_length);
			server.sin_family = hp->h_addrtype;
		}else{
			return(0);
		}
	}else{
		server.sin_family = AF_INET;
	}
	server.sin_port= (unsigned short) htons(port);
	sock = socket(AF_INET, SOCK_STREAM, 0);
	len = sizeof(server);
	if(connect(sock, (struct sockaddr *) &server, len) < 0) {
		return(0);
	}else{
		return(sock);
	}
}
